/*

lcdwho - display logged-in users on lcdproc server
Copyright (C) 1999 Dale Osowski

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
                
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
                                
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


Author can be reached at osowski@bigfoot.com

*/                                            
                                            
#include <utmp.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include "sockets.h"
#include "comm.h"

#define SHOW_SCREEN 128		// Normal screen priority
#define DONT_SHOW_SCREEN 255	// Nobody logged in:  don't show screen

#define DEBUG 1

struct w 			//User Info
{
	char name[UT_NAMESIZE+1];
	char host[UT_HOSTSIZE+1];
	char tty[UT_LINESIZE+1];//Use in future multi-screen mode?

	struct w *next;
};

struct p
{
	struct w *this;
};

char *version = VERSION;

void getwho(struct p *);
int count_users(struct w *);
void add2list(struct p *, struct utmp *);
void add2end(struct p *, struct utmp *);
void add2beg(struct p *, struct utmp *);
void freelist(struct p *);
int inlist(struct w *list, struct utmp *);
void client_init(int);
void sendstuff(int, struct w *, struct lcd_spec *);
void exit_program();
void useage();

int tsock;
int title = 127;
/* Default maximum title bar length.  This is user adjustable with the -t
   option, to allow user to decide the amount of scrolling that is done, or
   if any.
*/

int main(int argc, char *argv[])
{
	int sock, len, i;
	char buf[100];
	struct p *list;
	struct lcd_spec *mylcd;
	char hostname[100];
	int port = LCDPORT;
	hostname[0] = '\0';
	
	for(i=1; i<argc ;i++)
	{
		if(argv[i][0] == '-')
		switch(argv[i][1])
		{
			case 'h': //Set Hostname
				if((argv[i][2] == '\0')
				 && (argv[i+1] != NULL))
				  strcpy(hostname, argv[i+1]);
				else if(argv[i][2] != '\0')
				  strcpy(hostname, argv[i] +2);
				else {useage(); exit(0);}
				break;
			case 't': //Set maximum title bar length
				if((argv[i][2] == '\0')
				 && (argv[i+1] != NULL))
				  title = atoi(argv[i+1])+1;
				else if(argv[i][2] != '\0')
				  title = atoi(argv[i]+2)+1;
				else{useage(); exit(0);}
				break;
			case 'p': //Set LCDd port
				if((argv[i][2] == '\0')
				 && (argv[i+1] != NULL))
				  port = atoi(argv[i+1]);
				else if(argv[i][2] != '\0')
				  port = atoi(argv[i]+2);
				else{useage(); exit(0);}
				break;
			default: useage(); exit(0);
		}

	}
	if(hostname[0] == '\0')		// default to localhost
		strcpy(hostname, "localhost");
	if(title > 127) title = 127;	// in case user does something stupid
	list = malloc(sizeof(struct p));
	list->this = NULL;

	signal(SIGINT, exit_program);
	signal(SIGTERM, exit_program);
	signal(SIGHUP, exit_program);
	signal(SIGKILL, exit_program);
 
	mylcd = malloc(sizeof(struct lcd_spec));
	if(!(sock = lcd_init(hostname, port, mylcd)))
	{
		fprintf(stderr, "Oops, no connection\n");
		exit(0);
	}
	else
		printf("ok\n");
	tsock = sock;
	client_init(sock);
usleep(1000000);
	len = sock_recv(sock, buf, 100);
	while(1)
	{
	
		getwho(list);
printf("User count: %d\n", count_users(list->this));
printf("buf: %s\n", buf);
		if(count_users(list->this))
		{
			set_priority(sock, SHOW_SCREEN);
			if(0 == strncmp("listen whoscr", buf, 13))
			{
				sendstuff(sock, list->this, mylcd);
			}
		}
		else	//nobody is on, so no reason to show screen
		{
			set_priority(sock, DONT_SHOW_SCREEN);
		}
		usleep(500000);		// half second delay between updates
		len = sock_recv(sock, buf, 100);
		freelist(list);
	}
	return 0;
}

void getwho(struct p *list)	// let's find out who's on
{
	struct utmp *tmp;
	
	if(DEBUG) printf("In getwho()\n");
	list->this = NULL;
		
	setutent();
	tmp = getutent();
	while(tmp != NULL)
	{
		if(tmp->ut_type == USER_PROCESS && !inlist(list->this, tmp))
			add2list(list, tmp);
			
		tmp = getutent();
	}
		
	endutent();
}

				// Find out if we already found this user
				// (based on user && host fields)
int inlist(struct w *list, struct utmp *testval)
{
	struct w *current = list;
	if(DEBUG) printf("In inlist()\n");
	
	while(current != NULL)
	{

		if(0 == strcmp(testval->ut_user, current->name))
		{

			if(0 == strcmp(testval->ut_host, current->host))
			{
				return 1;
			}
			else if(0 == strcmp(current->host, "localhost") &&\
				0 == strlen(testval->ut_host))
				return 1;
		}
		current = current->next;
	}
	return 0;
}

void add2list(struct p *list, struct utmp *intmp)
{
	if(DEBUG) printf("in add2list()\n");
	if(strlen(intmp->ut_host) == 0 || strcmp(intmp->ut_host, ":0.0") == 0)
		add2end(list, intmp);
	else
		add2beg(list, intmp);
}

					// Users we don't care as much about
					// (Local users)
void add2end(struct p *list, struct utmp *intmp)
{
	struct w *temp;
	if(DEBUG) printf("In add2end\n");
	
	if(list->this == NULL)
	{
		list->this = malloc(sizeof(struct w));
		temp = list->this;
	}
	
	else
	{
		temp = list->this;
		while(temp->next != NULL)
			temp = temp->next;
		temp->next = malloc(sizeof(struct w));
		temp = temp->next;
	}
	
	strcpy(temp->name, intmp->ut_user);
	if(0 == strlen(intmp->ut_host))
		strcpy(temp->host, "localhost");
	else
		strcpy(temp->host, intmp->ut_host);
	strcpy(temp->tty, intmp->ut_line);
	temp->next = NULL;
}		

					// More interesting than local users
void add2beg(struct p *list, struct utmp *intmp)
{
	struct w *temp;
	temp = list->this;
	if(DEBUG) printf("In add2beg\n");

	
	list->this = malloc(sizeof(struct w));
	list->this->next = temp;
	temp = list->this;
	
	strcpy(temp->name, intmp->ut_user);
	if(0 == strlen(intmp->ut_host))
		strcpy(temp->host, "localhost");
	else
		strcpy(temp->host, intmp->ut_host);
	strcpy(temp->tty, intmp->ut_line);
}		



void freelist(struct p *list)		// Deallocate linked list
{
	struct w *temp;
	if(DEBUG) printf("In freelist\n");	
	while(list->this != NULL)
	{
		temp = list->this;
		list->this = list->this->next;
		free(temp);
	}
	list->this = NULL;
}


void client_init(int sock)		// Send client info to server
{
	int i;
	char temp[128];

	if(DEBUG) printf("In client_init()\n");
	
	sock_send_string(sock, "client_set name lcdwho\n");
	sock_send_string(sock, "screen_add whoscr\n");
	sock_send_string(sock, "widget_add whoscr ttl title\n");

	for(i=1; i<=3; i++)
	{
		// 3 usernames can fit on the screen
		sprintf(temp, "widget_add whoscr n%d string\n", i);
		sock_send_string(sock, temp);

		// host name for each username on screen
		sprintf(temp, "widget_add whoscr h%d string\n", i);
		sock_send_string(sock, temp);
	}
}

					// Tell the server what to say
void sendstuff(int sock, struct w *list, struct lcd_spec *mylcd)
{
	struct w *current = list;
	char str[256];
	char tmp[128];
	int i;
	char thishost[65];

	if(DEBUG) printf("In sendstuff\n");
	
	gethostname(thishost, 65);

	sprintf(tmp, "\"%d@", count_users(list));
	strcat(tmp, thishost);
	tmp[title] = '\0';
	strcat(tmp, "\"");
	sprintf(str, "widget_set whoscr ttl %s\n", tmp);
	sock_send_string(sock, str);
	
	// Display one user per line, minus the line for the title bar
	
	for(i=1; i <= (mylcd->lcd_hgt)-1 ; i++)
	{
		if(current != NULL)
		{
			str[0] = '\0';
			sprintf(str, "widget_set whoscr n%d 1 %d \"%s\"\n", \
				i, i+1, current->name);
			sock_send_string(sock, str);
			sprintf(tmp, "%s", current->host);
			tmp[11] = '\0';
			sprintf(str, "widget_set whoscr h%d 10 %d \"%s\"\n", \
				i, i+1, tmp);
			sock_send_string(sock, str);
			current = current->next;
		}
		else
		{
			str[0] = '\0';
			sprintf(str, "widget_set whoscr n%d 1 %d \"                   \"\n", \
				i, i+1);
			sock_send_string(sock, str);
			sprintf(str, "widget_set whoscr h%d 1 %d \"           \"\n", \
				i, i+1);
			sock_send_string(sock, str);
		
		}
	}
}

void exit_program()
{
	int len=0;
	char buf[100];
	len = sock_recv(tsock, buf, 100);

	sock_close(tsock);
	exit(0);
}


int count_users(struct w *list)
{
	if(list != NULL)
		return (1 + count_users(list->next));
	else
		return 0;
}

void useage(void)
{
	printf("lcdwho v%s\n", version);
	printf("Useage: lcdwho [-h hostname] [-p port] [-t length]\n\n");
	printf("-h hostname\tSend data to server on remote host \"hostname\"\n");
	printf("-p port\t\tServer is on port \"port\"\n");
	printf("-t length\tSet maximum length of title-bar text\n\n");
}

